// Exercice 2 de l'examen 1ere session 2012/2013 de NFA031
import java.util.*;

// Classe de définition d'un compte bancaire
//
public class CompteBancaire
{
    private double soldeInitial;           // Le solde initial
    private ArrayList<Ecriture> ecritures; // La liste des écritures

    // Constructeur
    //   - initialisation du sole initial et de la liste des écritures 
    //     en lisant le fichier et en décodant chacun des lignes
    //
    public CompteBancaire(String fichier) throws Exception  // Exception si erreur de décodage
    {
        // Création de la liste des écritures
        ecritures = new ArrayList<Ecriture>();

        // Lecture du fichier
        String[] lignes = Terminal.lireFichierTexte(fichier);

        // Decodage de chacune des lignes du fichier
        soldeInitial=0.0;
        for(int i=0;i<lignes.length;i++)
            {
                try{
                    if (i==0)
                        {
                            // Decodage de la 1ere ligne contenant le solde initial du compte
                            StringTokenizer strtok=new StringTokenizer(lignes[0]);
                            strtok.nextToken();
                            if (strtok.nextToken().equals("CREDIT"))
                                soldeInitial = Double.parseDouble(strtok.nextToken());
                            else
                                soldeInitial = - Double.parseDouble(strtok.nextToken());
                        }
                    else
                        {
                            // Decodage de chacune des écritures qui sont ajoutées dans
                            //   la liste des écritures
                            StringTokenizer strtok=new StringTokenizer(lignes[i]);
                            Ecriture ecr = new Ecriture(strtok.nextToken(),
                                                        Double.parseDouble(strtok.nextToken()),
                                                        strtok.nextToken());
                            ecritures.add(ecr);
                        }
                }
                // Quelque soit l'erreur d'interprétation des lignes du fichier texte
                //   pas le bon nombre de mots, la valeur n'est pas un double, ...
                // alors on remonte une exception en indiquant la ligne où se trouve l'erreur
                catch(Exception ex){
                    int num = i+1;
                    throw new Exception("Erreur dans le fichier "+fichier+" a la ligne "+num);
                }
            }
    }
    
    // Méthode qui calcule le solde du compte
    public double calculerSolde()
    {
        double solde = 0.0;
        for(Ecriture e:ecritures)
            {
                if (e.credit) solde=solde + e.valeur;
                if (e.debit)  solde=solde - e.valeur;
            }
        return solde + soldeInitial;
    }

    // Méthode qui calcule la somme des dépenses ou crédits d'une nature donnée
    public double calculerSommeNature(String nature)
    {
        double somme = 0.0;
        for(Ecriture e:ecritures)
            {
                if (e.nature.equals(nature))
                    {
                        if (e.credit) somme=somme + e.valeur;
                        if (e.debit)  somme=somme - e.valeur;
                    }
            }
        return somme;
    }

    // Programme principal qui calcul le solde de chacun des comptes d'une famille
    //  et le solde total de tous les comptes cumulés
    //
    public static void main(String... args)
    {
        double soldeTotal = 0.0; // Le solde total cupulé des compte

        // Pour chaque compte on va créé un CompteBancaire et 
        //  obtenir le solde
        // Si une erreur est remontée alors on trace l'erreur et 
        //   on passe au compte suivant
        //

        // Premier compte
        try{
            CompteBancaire c1 = new CompteBancaire("COMPTE_JOINT.txt");
            double solde = c1.calculerSolde();
            System.out.println("Le solde de COMPTE_JOINT est : "+ solde);
            soldeTotal=soldeTotal+solde;
        }catch(Exception ex)
            { System.out.println(ex.getMessage()); }

        // Deuxieme compte
        try{
            CompteBancaire c1 = new CompteBancaire("COMPTE_EPARGNE.txt");
            double solde = c1.calculerSolde();
            System.out.println("Le solde de COMPTE_EPARGNE est : "+ solde);
            soldeTotal=soldeTotal+solde;
        }catch(Exception ex)
            { System.out.println(ex.getMessage()); }

        // Troisieme compte
        try{
            CompteBancaire c1 = new CompteBancaire("LIVRET_A.txt");
            double solde = c1.calculerSolde();
            System.out.println("Le solde de LIVRET_A est : "+ solde);
            soldeTotal=soldeTotal+solde;
        }catch(Exception ex)
            { System.out.println(ex.getMessage()); }

        // Affichage du solde total
        System.out.println("Le solde total est de : "+ soldeTotal);
    }
}

// Classe interne qui définit une écriture d'un compte bancaire
class Ecriture
{
    boolean credit;  // true si l'écriture est un crédit sinon false
    boolean debit;   // true si l'écriture est un débit sinon false
    double  valeur;  // valeur de l'écriture
    String  nature;  // nature de l'écriture

    // Constructeur
    //
    public Ecriture(String debitCredit,double valeur,String nature)
    {
        this.credit = false;
        this.debit = false;
        if (debitCredit.equals("CREDIT")) this.credit = true;
        if (debitCredit.equals("DEBIT"))  this.debit  = true;
        this.valeur=valeur;
        this.nature=nature;
    }
}

        
